/*
cprinter.h
Copyright (C)2002 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2002-2004 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/

#ifndef _CPRINTER_H_
#define _CPRINTER_H_

#include <gtk--/main.h>
#include <ctransitems.h>

#ifdef _USB_DRIVER
#include <usb.h>
#endif

#include "postdev.h"
#include "pos.h"

/**
 * @defgroup clientlib Client Libraries
 * @{
 *
 */

namespace BhPos{
	
/**
 * @defgroup printer Printer
 * @{
 *
 * @ingroup clientlib
 * @{	
 */

#define PRINTER_PARAM_HDRMSG		1
#define PRINTER_PARAM_FTRMSG		2
#define PRINTER_PARAM_SLIPAREA	3

#define PRINTER_FEATURE_BASE				0x00000001
#define PRINTER_FEATURE_MICR			  (PRINTER_FEATURE_BASE)
#define PRINTER_FEATURE_MICRE13B	     (PRINTER_FEATURE_BASE << 1)
#define PRINTER_FEATURE_MICRCMC7	     (PRINTER_FEATURE_BASE << 2)
#define PRINTER_FEATURE_CUSTDISP	 	  (PRINTER_FEATURE_BASE << 3)
#define PRINTER_FEATURE_CASHDRAWER	  (PRINTER_FEATURE_BASE << 4)

/*
#define PRINTER_DEVSTATUS_CLOSED		0
#define PRINTER_DEVSTATUS_OPEN		1
#define PRINTER_DEVSTATUS_READERR	2
#define PRINTER_DEVSTATUS_WRITEERR 3
*/

/* }@
 *
 * @ingroup printer
 * @{
 */
 
enum BhPrinterStatusMethod{
	PRN_STATUSMETHOD_NORMAL,
	PRN_STATUSMETHOD_REALTIME,
	PRN_STATUSMETHOD_MICRREALTIME,
	PRN_STATUSMETHOD_ASB
};
/* }@
 *
 * @ingroup printer
 * @{
 */
#define PRINTER_STATUS_UNKNOWN			 0x00000000
#define PRINTER_GETSTATUS_PRINTER		 0x00000001
#define PRINTER_GETSTATUS_OFFLINE		(0x00000001 << 1)
#define PRINTER_GETSTATUS_ERROR		   (0x00000001 << 2)
#define PRINTER_GETSTATUS_PAPERROLL	   (0x00000001 << 3)
#define PRINTER_GETSTATUS_SLIP			(0x00000001 << 4)
#define PRINTER_GETSTATUS_VALIDATION	(0x00000001 << 5)
#define PRINTER_GETSTATUS_MICR			(0x00000001 << 6)

// from status n=1
#define PRINTER_STATUS_KICK		 	    0x00000001
#define PRINTER_STATUS_ONLINE				(0x00000001 << 1)
#define PRINTER_STATUS_WOLRECOVERY     (0x00000001 << 3)
#define PRINTER_STATUS_PAPERFEED	      (0x00000001 << 3)
// from status n=2
#define PRINTER_STATUS_COVER   		   (0x00000001 << 4)
#define PRINTER_STATUS_OLPAPERFEED     (0x00000001 << 5)
#define PRINTER_STATUS_PAPEREND	      (0x00000001 << 6)
#define PRINTER_STATUS_ERROR	 	      (0x00000001 << 7)
// from status n=3
#define PRINTER_STATUS_MECHERROR  	   (0x00000001 << 8)
#define PRINTER_STATUS_ACUTTERERROR    (0x00000001 << 9)
#define PRINTER_STATUS_UNRECERROR	   (0x00000001 << 10)
#define PRINTER_STATUS_AUNRECERROR	   (0x00000001 << 11)
// from status n=4
#define PRINTER_STATUS_NEAREND    	   (0x00000001 << 12)
#define PRINTER_STATUS_PAPERPRESENT    (0x00000001 << 13)
// from status n=5
#define PRINTER_STATUS_SLIPSELECT	   (0x00000001 << 14)
#define PRINTER_STATUS_SLIPWAIT   	   (0x00000001 << 15)
#define PRINTER_STATUS_SLIPTOF    	   (0x00000001 << 16)
#define PRINTER_STATUS_SLIPBOF    	   (0x00000001 << 17)
// from status n=6
#define PRINTER_STATUS_VALSELECT	      (0x00000001 << 18)
#define PRINTER_STATUS_VALWAIT    	   (0x00000001 << 19)
#define PRINTER_STATUS_VALTOF     	   (0x00000001 << 20)
#define PRINTER_STATUS_VALBOF     	   (0x00000001 << 21)
// from status mirc
#define PRINTER_STATUS_MICRSELECT	   (0x00000001 << 22)
#define PRINTER_STATUS_MICRWAIT    	   (0x00000001 << 23)
#define PRINTER_STATUS_MICRTOF       	(0x00000001 << 24)
#define PRINTER_STATUS_MICRBOF     	   (0x00000001 << 25)

#define PRINTER_GS_PAPERROLL_NEAREND  1
#define PRINTER_GS_PAPERROLL_PAPER    2 
#define PRINTER_GS_PAPERROLL_TOF      3
#define PRINTER_GS_PAPERROLL_BOF      4
#define PRINTER_GS_DRAWER_OPEN        5
#define PRINTER_GS_SLIPPAPER_PRESENT  6

#define PRINTER_ABS_WAITINGONINSERT   1
#define PRINTER_ABS_SLIPINTSERTED	  2
#define PRINTER_ABS_WAITINGONEJECT	  4
#define PRINTER_ABS_SLIPEJECTED  	  8

#define PRINTER_BUFFER_STATUS_INPUT	  1
#define PRINTER_BUFFER_STATUS_OUTPUT  2

/* }@
 *
 * @ingroup printer
 * @{
 */
 
typedef struct tagPrintArea{
	int unit;
	int sx, sy, dx, dy;
}PrintArea;

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagHeaderPrint{
	int  transID;
	int  UserID;
	int  StationID;
	char datetime[18];
	char msg[126];
}HeaderPrint;

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagTransSummary{
   int   NumItems;
   float Subtotal;
   float Discount;
   float Tax;
   float Total;
}TransSummary;

typedef struct tagChargeSummary{
   float old_balance;
   float new_balance;
   float limit;
}ChargeSummary;


/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhChequeLayoutConversion{
   CHK_LAYOUT_PAGE,
   CHK_LAYOUT_MM,
   CHK_LAYOUT_INCH
};

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagcoord{
	int x;
	int y;
}coord;

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagbcoord{
	unsigned char x[2];
	unsigned char y[2];
}bcoord;

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagrcoord{
	float x;
	float y;
}rcoord;

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagChequeLayout{
	int    x_dpi;
	int    y_dpi;
	rcoord size;
	rcoord date;
	rcoord year;
	rcoord number_amount;
	rcoord number_word1;
	rcoord number_word1_end;
	rcoord number_word2;
	rcoord payto;
}ChequeLayout;

/* }@
 *
 * @ingroup printer
 * @{
 */
typedef struct tagChequeUnitLayout{
	coord size;
	coord date;
	coord year;
	coord number_amount;
	coord number_word1;
	coord number_word1_end;
	coord number_word2;
	coord payto;
}ChequeUnitLayout;

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagChequeByteUnitLayout{
	bcoord size;
	bcoord date;
	bcoord year;
	bcoord number_amount;
	bcoord number_word1;
	int	 num_word1_chars;
	bcoord number_word2;
	bcoord payto;
}ChequeByteUnitLayout;

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagChequePrint{
	bool	 preLoaded;
   char   date[32];
	char   year[8];
	char   payto[64];
	char	 MajAmount[32];
	char	 MinAmount[32];
	double amount;
	char   endAccNum[32];
}ChequePrint;

/* }@
 *
 * @ingroup printer
 * @{
 */

#ifndef _DEFAULT_CHEQUE
#define _DEFAULT_CHEQUE
static ChequeLayout StandardChequeUSPersonal={0,0,{152.4,69.83},{81,22},{109,22},{23,31},{114,31},{7,39},{12,58}};
static ChequeLayout StandardChequeUKPersonal={0,0,{152,71},{100,19},{145,19},{112,41},{10,42},{100,42},{10,51},{17,33}};

#define DEFAULT_NUM_CHK 2

static ChequeLayout BhStdCheque[DEFAULT_NUM_CHK]={
   StandardChequeUSPersonal,
   StandardChequeUKPersonal
};
#endif

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagCreditPrint{
	bool	 bSigline;
	char   header[32];
   char   datetime[32];
	char   trans[16];
	char   station[16];
	char   user[16];
	char	 auth[32];
	char	 cardtype[16];
	char	 cardnum[32];
	double amount;
}CreditPrint;

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrinterFont{
   PF_PARAM_NUMBER,
   PF_PARAM_UNIT,
   PF_PARAM_XSIZE,
   PF_PARAM_YSIZE,
   PF_PARAM_CHARLINE
};

/* }@
 *
 * @ingroup printer
 * @{
 */

class CPrinterFont
{
private:
   int unit;
   int xsize, ysize;
   int charline;
   
public:
   CPrinterFont();
   CPrinterFont(CPrinterFont const&);
   CPrinterFont(int u, int x, int y, int cl);
   ~CPrinterFont();
   void operator=(CPrinterFont const&);
   bool operator==(CPrinterFont const&);
   bool operator<(CPrinterFont const&);
   
   void SetParameter(int, void *);
   void GetParameter(int, void *);
};

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrinterMechType{
   PM_TYPE_UNKNOWN=0,
   PM_TYPE_IMPACT,
   PM_TYPE_THERMAL,
   PM_TYPE_INKJET,
   PM_TYPE_LASER,
   PM_TYPE_WAXSUB
};

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrinterMechPaperType{
   PM_PAPERTYPE_UNKNOWN=0,
   PM_PAPERTYPE_RECIEPT,
   PM_PAPERTYPE_SLIP,
   PM_PAPERTYPE_JOURNAL,
   PM_PAPERTYPE_ENDORSE,
   PM_PAPERTYPE_LABEL,
   PM_PAPERTYPE_OTHER
};

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrinterMechCutterType{
   PM_CUTTERTYPE_NONE     = 1,
   PM_CUTTERTYPE_FULL     = 2,
   PM_CUTTERTYPE_PARTIAL  = 4,
   PM_CUTTERTYPE_AUTO     = 8
};

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrinterMechParam{
   PM_PARAM_TYPE,
   PM_PARAM_PAPERTYPE,
   PM_PARAM_XMOTIONUNIT,
   PM_PARAM_YMOTIONUNIT,
   PM_PARAM_XPAPERSIZE,
   PM_PARAM_YPAPERMINSIZE,
   PM_PARAM_YPAPERMAXSIZE,
   PM_PARAM_HASPAGEMODE,
   PM_PARAM_XDPI,
   PM_PARAM_YDPI,
   PM_PARAM_NUMFONTS,
   PM_PARAM_BIDIR,
   PM_PARAM_FONT,
   PM_PARAM_FONTSIZE,
   PM_PARAM_XCHARFONT,
   PM_PARAM_NUMCOLS,
   PM_PARAM_COLOUR,
   PM_PARAM_CUTTER,
   PM_PARAM_DEFCHARLINE
};

/* }@
 *
 * @ingroup printer
 * @{
 */

class CPrintMechanism{
private:
   BhPrinterMechType type; 
   BhPrinterMechPaperType papertype;
   int xpapersize;
   int ypaperminsize, ypapermaxsize;
   int xmotionunit, ymotionunit; 
   int xdpi, ydpi;
   
   bool bHasPageMode;
   bool bBidrectional;
   BhPrinterMechCutterType  cuttertype;
   
   std::list<CPrinterFont>   fonts;
   
   int curr_font;
   int numcolours;

protected:

public:
   CPrintMechanism();
   CPrintMechanism(CPrintMechanism const&);
   ~CPrintMechanism();
   void operator=(CPrintMechanism const&);
   bool operator==(CPrintMechanism const&);
   bool operator<(CPrintMechanism const&);
   
   void SetParameter(int, void *);
   void GetParameter(int, void *);
   int AddFont(CPrinterFont&);
};

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagPrintCollect{
	unsigned char num;
	unsigned int  mask;
	unsigned int  data;
}PrintCollect;

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrinterProtocol{
   ESCPOS,
   DP8340,
   STAR,
   CITIZEN,
   PCOS,
   PSTEMPLATE
};

/* }@
 *
 * @ingroup printer
 * @{
 */

enum MICRFontType{
   E13B=0,
   CMC7
};

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrintTimerType{
   PRNTIMER_SLIPINSERT,
   PRNTIMER_SLIPEJECT,
	PRNTIMER_CHQINSERT,
	PRNTIMER_CHQEJECT,
	PRNTIMER_MICRINSERT,
	PRNTIMER_MICREJECT
};

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrintPaperType{
   PRN_PAPERTYPE_UNKNOWN=0,
   PRN_PAPERTYPE_RECIEPT,
   PRN_PAPERTYPE_SLIP,
   PRN_PAPERTYPE_JOURNAL,
   PRN_PAPERTYPE_ENDORSE,
   PRN_PAPERTYPE_LABEL,
   PRN_PAPERTYPE_MICR,
	PRN_PAPERTYPE_CHEQUE
};

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrintType{
   PRNCMD_SETUP,
   PRNCMD_INTIALIZE,
   PRNCMD_NVBITMAP,
   PRNCMD_PREPRINT,
   PRNCMD_HEADER,
   PRNCMD_ITEM,
   PRNCMD_TRANSSUMMARY,
	PRNCMD_PAYITEM,
   PRNCMD_PAYSUMMARY,
	PRNCMD_CHANGE,
   PRNCMD_CHARGE,
   PRNCMD_FOOTER,
   PRNCMD_POSTPRINT,
   PRNCMD_SLIP_PREPRINT,
   PRNCMD_SLIP_HEADER,
   PRNCMD_SLIP_ITEM,
   PRNCMD_SLIP_FOOTER,
   PRNCMD_SLIP_POSTPRINT,
	PRNCMD_SLIP_CANCEL,
   PRNCMD_LABEL_PREPRINT,
   PRNCMD_LABEL_ITEM,
   PRNCMD_LABEL_POSTPRINT,
   PRNCMD_JRN_PREPRINT,
   PRNCMD_JRN_ITEM,
   PRNCMD_JRN_POSTPRINT,
   PRNCMD_END_PREPRINT,
   PRNCMD_END_ITEM,
   PRNCMD_END_POSTPRINT,
   PRNCMD_CHQ_PREPRINT,
   PRNCMD_CHQ_ITEM,
   PRNCMD_CHQ_POSTPRINT,
   PRNCMD_BARCODE,
   PRNCMD_CDISPPRESALE,
   PRNCMD_CDISPPOSTSALE,
	PRNCMD_CDISPIDLE,
   PRNCMD_CASHD,
   PRNCMD_MICR_PREREAD,
   PRNCMD_MICR_READ,
   PRNCMD_MICR_STATUS,
   PRNCMD_MICR_DATA,
   PRNCMD_MICR_POSTREAD,
	PRNCMD_MICR_CANCEL,
	PRNCMD_CREDIT,
	PRNCMD_INVOICE,
   PRNCMD_TEMPLATE,
   PRNCMD_RAW
};

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrintParam{
   PRN_PARAM_PAGEMODE,
   PRN_PARAM_DEFXRES,
   PRN_PARAM_DEFYRES,
   PRN_PARAM_CASHDRAWER,
   PRN_PARAM_CUSTDISP,
   PRN_PARAM_MICR,
   PRN_PARAM_MICRFONTYPE,
   PRN_PARAM_RECCHARLINE,
   PRN_PARAM_RECFONT,
   PRN_PARAM_JRNCHARLINE,
   PRN_PARAM_JRNFONT,
   PRN_PARAM_SLIPCHARLINE,
   PRN_PARAM_SLIPFONT,
   PRN_PARAM_CHEQUE,
   PRN_PARAM_CHEQUEROTATE,
   PRN_PARAM_CHEQUEENDORSE,
   PRN_PARAM_CUSTDISPTYPE,
   PRN_PARAM_CUSTDISPX,
   PRN_PARAM_CUSTDISPY,
	PRN_PARAM_NVRAM,
	PRN_PARAM_NVRAMSIZE,
	PRN_PARAM_BARCODE,
	PRN_PARAM_AUTOCUTTER,
   PRN_PARAM_CREDITSLIP,
	PRN_PARAM_AUTOPOLE,
	PRN_PARAM_NVRAM_HDRLOGO,
   PRN_PARAM_NVRAM_FTRLOGO,
   PRN_PARAM_NVRAM_SLIPLOGO,
   PRN_PARAM_NVRAM_HDRMSG,
   PRN_PARAM_NVRAM_FTRMSG,
   PRN_PARAM_NVRAM_RECHDRLAYOUT,
   PRN_PARAM_NVRAM_ITEMLAYOUT,
   PRN_PARAM_NVRAM_PAYSUMLAYOUT,
   PRN_PARAM_NVRAM_TRANSSUMLAYOUT,
   PRN_PARAM_NVRAM_SLIPHDRLAYOUT,
   PRN_PARAM_NVRAM_SLIPITEMLAYOUT,
   PRN_PARAM_NVRAM_SLIPPAYSUMLAYOUT,
   PRN_PARAM_NVRAM_SLIPTRANSSUMLAYOUT,
   PRN_PARAM_NVRAM_JRNHDR,
   PRN_PARAM_NVRAM_JRNITEM,
   PRN_PARAM_NVRAM_JRNFTR,
   PRN_PARAM_NVRAM_CHPAYTO,
   PRN_PARAM_NVRAM_MAJAMOUNT,
   PRN_PARAM_NVRAM_MINAMOUNT,
   PRN_PARAM_NVRAM_CDIDLEMSG,
   PRN_PARAM_NVRAM_CDPRESALE,
   PRN_PARAM_NVRAM_CDPOSTSALE,
   PRN_PARAM_NVRAM_ROLLCNT,
   PRN_PARAM_NVRAM_MICRCNT
};

/* }@
 *
 * @ingroup printer
 * @{
 */

enum BhPrinterNvRam{
   PRN_NVRAM_HDRLOGO=1,
   PRN_NVRAM_FTRLOGO,
   PRN_NVRAM_SLIPLOGO,
   PRN_NVRAM_HDRMSG=40,
   PRN_NVRAM_FTRMSG,
   PRN_NVRAM_RECHDRLAYOUT,
   PRN_NVRAM_ITEMLAYOUT,
   PRN_NVRAM_PAYSUMLAYOUT,
   PRN_NVRAM_TRANSSUMLAYOUT,
   PRN_NVRAM_SLIPHDRLAYOUT,
   PRN_NVRAM_SLIPITEMLAYOUT,
   PRN_NVRAM_SLIPPAYSUMLAYOUT,
   PRN_NVRAM_SLIPTRANSSUMLAYOUT,
   PRN_NVRAM_JRNHDR,
   PRN_NVRAM_JRNITEM,
   PRN_NVRAM_JRNFTR,
   PRN_NVRAM_CHPAYTO,
   PRN_NVRAM_MAJAMOUNT,
   PRN_NVRAM_MINAMOUNT,
   PRN_NVRAM_CDIDLEMSG,
   PRN_NVRAM_CDPRESALE,
   PRN_NVRAM_CDPOSTSALE,
   PRN_NVRAM_ROLLCNT,
   PRN_NVRAM_MICRCNT
};

/* }@
 *
 * @ingroup printer
 * @{
 */

#define PRN_INSTREAM_POLLMICR   1
#define PRN_INSTREAM_POLLCHEQUE 2

#define PRN_NOMOREDATA	40
#define PRN_MOREDATA	   41

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagPrnNVMsg{
	unsigned char header;
	unsigned char ident;
	unsigned char status;
	char			  data[81];
}PrnNVMsg;

/* }@
 *
 * @ingroup printer
 * @{
 */

typedef struct tagPrnNVRam{
	unsigned char key1;
	unsigned char key2;
	unsigned char size;
	unsigned char status;
	char			  data[1];
}PrnNVRam;

/* }@
 *
 * @ingroup printer
 * @{
 */

class CPrinter
{
private:
	int               printer_status;
	BhPrinterProtocol protocol;
	int               dev;
#ifdef _USB_DRIVER
   usb_dev_handle   *usb_dev;
#endif
	bool				   bRealTimeStatusIssued;
	bool				   bWaitingPaperInsert;
   bool              bCreditCut;
   bool              bCreditSlip;
	
	unsigned int reccharline;
   unsigned int jrncharline;
   unsigned int slipcharline;
   unsigned int recfont;
   unsigned int jrnfont;
   unsigned int slipfont;
	unsigned int charsize;
   unsigned int max_line;

	char *linebuffer;
   
   // features for real time use
   bool bCustomerDisplay;
	bool bAutoCustomerDisplay;
   int  custdisp_type;
   bool bCashDrawer;
   bool bPageMode;
	bool bNVRAM;
	bool bMICR;
   bool bAutoCutter;
	bool bBarcode;
   bool bChequePrint;
   bool bChequeRotate;
   bool bEndorsement;
   
   unsigned int  nvramsize;
   unsigned int  x_res;
   unsigned int  y_res;
   unsigned int  custdisp_x;
   unsigned int  custdisp_y;
   MICRFontType  mircft;
	ChequeByteUnitLayout chklayout;
   
   int          tbuffsize;
   char         tbuff[128];
   
   unsigned char  tprinttype, num_copies;
   unsigned char  num_top, num_end, num_cont, num_sheets;
   unsigned char  top_yinc, end_yinc, cont_yinc;
   char          *topbuff, *contbuff, *endbuff;
   int            top_len, end_len, cont_len;
   FILE          *tp, *cp, *ep;
	
	bool bAbsWatcher;

protected:
   BhPostDevice id;
   
   int Output(int, void *);
	int Input(int, void *, bool);
	
   int Setup(char *d, int len);
	int Initialize(char *d, int len);
	int PrePrint(char *d, int len);
	int PrintHeader(HeaderPrint *);
	int PrintItem(CSaleItem *);
	int PrintTransSummary(TransSummary *);
	int PrintPayItem(int, CPayItem *);
	int PrintChange(double *);
   int PrintCharge(ChargeSummary *);
	int PrintPaySummary(CTransItems *);
	int PrintFooter(char *d, int len);
	int PostPrint(char *d, int len);
   int SlipPrePrint(char *d, int len);
   int SlipHeader(char *, int);
	int SlipItem(CSaleItem *);
	int SlipFooter(char *, int);
	int SlipPostPrint(char *d, int len);
	int SlipCancel();
   int LabelPrePrint(char *d, int len);
	int LabelItem(CSaleItem *);
	int LabelPostPrint(char *d, int len);
   int JournalPrePrint(char *d, int len);
	int JournalItem(CSaleItem *);
	int JournalPostPrint(char *d, int len);
   int EndorsementPrePrint(char *d, int len);
	int EndorsementItem(CTransItems *);
	int EndorsementPostPrint(char *d, int len);
	int ChequePrePrint(char *d, int len);
	int ChequeItem(ChequePrint *);
	int ChequePostPrint(char *d, int len);
	int PrintCredit(CreditPrint *);
	int MicrPreread(char *d, int len);
	int MicrRead();
	unsigned char MicrStatus();
	int MicrCancel();
   int MicrData(BhChequeFormattedData *);
	int MicrPostread(char *d, int len);
	int PrintBarcode();
	int PrintCustDisp(unsigned char disp, int x, int y, CSaleItem *msg);
	int PrintCustDisp(char *);
	int FireCashDrawer(int drawer);
	int PaperCut(unsigned char type, unsigned char feed);
	
	int  ChequeUnitToByte(ChequeUnitLayout *, ChequeByteUnitLayout *);
   void SpellMoney(double, char *, char *, char *);
	void JustifyLine(int, char *, char *);
   
   int GetPageVariables(int);
   int ReplaceVariableInstances(int f, int numc, CSaleItem &si);
   int ReplaceFixedInstances(int f, int numc, int sheetnum, int sheetof, CTransItems &trans, BhCustomerItem *cust);
   int ReplaceTag(int f, char *tag, char *rep);
   
#ifdef _USB_DRIVER
   int usbFind();
#endif

public:
   CPrinter(int);
#ifdef _USB_DRIVER
   CPrinter(int);
#endif
	~CPrinter();
	
	bool PrinterExists();
	int  GetDevice() { return dev; }
	
	int  SetParam(BhPrintParam, void *);
	int  GetParam(BhPrintParam, void *);
	int  QuickParse(char *s);
	
	int ReadNVRAM(char, char, char *);
   int WriteNVRam(int, char, char, unsigned char *);
	int NVRam(bool bUsed);
   
   int OpenTemplate(char *toppage, char *contpage, char *endpage);

	void TabSet(int flags);
	int  ChequeLayoutToUnit(ChequeLayout *, ChequeUnitLayout *, BhChequeLayoutConversion);
	int  ChequeLayoutToUnit(ChequeLayout *, BhChequeLayoutConversion);
	int  PrintZTape(HeaderPrint *, BhServiceTransSum *);

	int Print(BhPrintType, int, void *);
   int PrintTemplate(CTransItems &trans, BhCustomerItem *cust, int numcopies);
	unsigned int GetPrinterStatus(unsigned int type, BhPrinterStatusMethod method);
	int GetPaperStatus(BhPrintPaperType type, bool bPresent);
	void Lock();
	void Release();
};

namespace Parser{

#define PRNLO_HAVEPAGEMODE 1
#define PRNLO_HEADER	 	   2
#define PRNLO_ITEM	 	   3
#define PRNLO_FOOTER	 	   4
	
enum BhPageAttribute{
   PRN_PAGE_TOPMARGIN,
   PRN_PAGE_BOTTOMMARGIN,
   PRN_PAGE_LEFTMARGIN,
   PRN_PAGE_RIGHTMARGIN,
   PRN_PAGE_LENGTHLINE,
   PRN_PAGE_LENGTHUNIT,      //! page length in millimetres
   PRN_PAGE_ORIGINX,         //! set origin of x in points
   PRN_PAGE_ORIGINY,         //! set origin of y in points
   PRN_PAGE_SIZEX,           //! set size of x in points
   PRN_PAGE_SIZEY,           //! set size of y in points
   PRN_PAGE_ENDX,            //! set bottom right coordinate of x in points
   PRN_PAGE_ENDY             //! set bottom right coordinate of y in points
};

enum BhCharAttribute{
   PRN_CHAR_SET        = 1,      // character set
   PRN_CHAR_DOTPITCH   = 2,
   PRN_CHAR_SPACING    = 4,
   PRN_CHAR_MAGNIFIY   = 8
};

enum BhPrintAttribute{
   PRN_ATTRIB_EMPHASIS   = 1,
   PRN_ATTRIB_UNDERLINE  = 2,
   PRN_ATTRIB_REVERSE    = 4,
   PRN_ATTRIB_DBLSTRIKE  = 8,
   PRN_ATTRIB_UPSIDEDOWN = 16,
   PRN_ATTRIB_ROTATE     = 32
};

}

/*	}@ 
	}@ */
}

/* }@ */

#endif	// _CPRINTER_H_
